'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DOCUMENT_PROPERTY_NAME = exports.NS_PREFIX = undefined;
exports.getPartPropertyName = getPartPropertyName;
exports.getPart = getPart;
exports.getPartNSPropertyName = getPartNSPropertyName;
exports.deepGet = deepGet;
exports.deepSet = deepSet;
exports.createDocument = createDocument;
exports.createRecordData = createRecordData;
exports.getDocument = getDocument;
exports.attributePropertiesToTop = attributePropertiesToTop;
exports.prepareForSending = prepareForSending;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var numericPattern = /^[0-9]$/;

var NS_PREFIX = exports.NS_PREFIX = 'ns2';
var DOCUMENT_PROPERTY_NAME = exports.DOCUMENT_PROPERTY_NAME = 'document';

function getPartPropertyName(partName) {
  return NS_PREFIX + ':' + partName;
}

function getPart(cspaceDocument, partName) {
  return cspaceDocument.get(getPartPropertyName(partName));
}

function getPartNSPropertyName() {
  return '@xmlns:' + NS_PREFIX;
}

/**
 * Deeply get a value in an Immutable.Map. This is similar to Immutable.Map.getIn, but differs in
 * one way:
 *
 * When a key of '0' is encountered, and that key is used to index a data item that is not a List,
 * the data item itself is returned. This accommodates data in which a list containing a single
 * item may be represented by that one item.
 */
function deepGet(data, path) {
  if (!Array.isArray(path) || path.length === 0) {
    throw new Error('path must be a non-empty array');
  }

  if (!data) {
    return undefined;
  }

  var _path = _toArray(path),
      key = _path[0],
      rest = _path.slice(1);

  var value = void 0;

  if ((key === '0' || key === 0) && !_immutable2.default.List.isList(data)) {
    // Allow a key of 0 to refer to a single non-list value.
    value = data;
  } else {
    value = data.get(key);
  }

  if (!value || rest.length === 0) {
    return value;
  }

  return deepGet(value, rest);
}

/**
 * Deeply set a value in an Immutable.Map. This is similar to Immutable.Map.setIn, but differs in
 * two ways:
 *
 * When a non-existent key is encountered in the middle of a path, this function may create a List
 * or a Map at that location, depending on the key. If the key is a numeric string, a List is
 * created. Otherwise, a Map is created. Immutable.Map.setIn always creates a Map.
 *
 * This function also promotes an existing singular (non-List) item to a List, if any numeric key
 * is applied to it.
 */
function deepSet(data, path, value) {
  if (!Array.isArray(path) || path.length === 0) {
    throw new Error('path must be a non-empty array');
  }

  var _path2 = _toArray(path),
      key = _path2[0],
      rest = _path2.slice(1);

  var isKeyNumeric = numericPattern.test(key);

  var normalizedData = void 0;

  if (data) {
    if (isKeyNumeric && !_immutable2.default.List.isList(data)) {
      // Promote a single (non-list) value into a list when a numeric key is supplied.
      normalizedData = _immutable2.default.List.of(data);
    } else {
      normalizedData = data;
    }
  } else if (isKeyNumeric) {
    normalizedData = _immutable2.default.List();
  } else {
    normalizedData = _immutable2.default.Map();
  }

  var resolvedValue = rest.length === 0 ? value : deepSet(normalizedData.get(key), rest, value);

  return normalizedData.set(key, resolvedValue);
}

/**
 * Create a skeletal document for a given CollectionSpace service.
 */
function createDocument(recordTypeConfig) {
  var serviceConfig = recordTypeConfig.serviceConfig;


  var cspaceDocument = {
    '@name': serviceConfig.documentName
  };

  var parts = serviceConfig.parts;

  Object.keys(parts).forEach(function (part) {
    cspaceDocument[getPartPropertyName(part)] = _defineProperty({}, getPartNSPropertyName(), parts[part]);
  });

  return _immutable2.default.fromJS(cspaceDocument);
}

/**
 * Create a skeletal data record for a given CollectionSpace service.
 */
function createRecordData(recordTypeConfig) {
  return _immutable2.default.Map().set(DOCUMENT_PROPERTY_NAME, createDocument(recordTypeConfig));
}

/**
 * Get the document from the data record.
 */
function getDocument(data) {
  return data.get(DOCUMENT_PROPERTY_NAME);
}

/**
 * Comparator function to sort properties that represent XML attributes and namespace declarations
 * (those that start with '@') to the top.
 */
function attributePropertiesToTop(propertyNameA, propertyNameB) {
  var firstCharA = propertyNameA.charAt(0);
  var firstCharB = propertyNameB.charAt(0);

  if (firstCharA === firstCharB) {
    return 0;
  }

  if (firstCharA === '@') {
    return -1;
  }

  if (firstCharB === '@') {
    return 1;
  }

  return 0;
}

/**
 * Prepare record data for POST or PUT to the CollectionSpace REST API. Document parts that may
 * be present in data retrieved from the REST API, but that do not need to be present in data sent
 * to the API, are removed. In the remaining parts, properties beginning with '@', which represent
 * XML attributes and namespace declarations, are moved to the top. This is required by the REST
 * API in order to properly translate the payload to XML.
 */
function prepareForSending(data) {
  // Filter out the core schema and account information parts.

  var cspaceDocument = data.get(DOCUMENT_PROPERTY_NAME).filter(function (value, key) {
    return key !== NS_PREFIX + ':collectionspace_core' && key !== NS_PREFIX + ':account_permission';
  });

  // For each remaining part, move XML attribute and namespace declaration properties (those
  // that start with @) to the top, since the REST API requires this.

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = cspaceDocument.keys()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      if (key.charAt(0) !== '@') {
        var part = cspaceDocument.get(key);
        var sortedPart = part.sortBy(function (value, name) {
          return name;
        }, attributePropertiesToTop);

        cspaceDocument = cspaceDocument.set(key, sortedPart);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return data.set(DOCUMENT_PROPERTY_NAME, cspaceDocument);
}