'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isPending = isPending;
exports.getResult = getResult;
exports.getError = getError;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _search = require('../actions/search');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var searchKey = function searchKey(searchDescriptor) {
  return JSON.stringify(searchDescriptor);
};

var handleSearchStarted = function handleSearchStarted(state, action) {
  var searchDescriptor = action.meta;
  var key = searchKey(searchDescriptor);

  if (state.hasIn(['byKey', key]) && !state.getIn(['byKey', key, 'error'])) {
    // Already have a non-error search state for this search descriptor. Do nothing.

    return state;
  }

  var mostRecentKey = state.get('mostRecentKey');

  var updatedState = state;
  var result = null;

  if (mostRecentKey) {
    var mostRecentSearchState = state.getIn(['byKey', mostRecentKey]);
    var mostRecentSearchDescriptor = mostRecentSearchState.get('descriptor');

    if (mostRecentSearchDescriptor.get('recordType') !== searchDescriptor.recordType || mostRecentSearchDescriptor.get('vocabulary') !== searchDescriptor.vocabulary || mostRecentSearchDescriptor.getIn(['searchQuery', 'kw']) !== searchDescriptor.searchQuery.kw || mostRecentSearchDescriptor.getIn(['searchQuery', 'size']) !== searchDescriptor.searchQuery.size) {
      // Something other than the page number and/or sort direction changed from the last search.
      // Clear out stored search states. (If only the page number and/or sort direction changed,
      // keep the previous states around as a cache so that flipping between pages or sort
      // directions will be fast.)

      // TODO: Might want to clear states when sort direction changes. Or just keep the current
      // page, so flipping sort dir on the current page will still be fast.

      updatedState = state.set('byKey', _immutable2.default.Map());
    } else {
      // Only the page number and/or sort direction changed from the last search. Seed the search
      // state with a blank result that has the same number of total items as the last search, and
      // the page num/page size of this search. This allows a blank result table to be rendered
      // while the search is pending, preventing a big flash of empty content.

      var totalItems = mostRecentSearchState.getIn(['result', 'ns2:abstract-common-list', 'totalItems']);

      if (totalItems) {
        result = _immutable2.default.fromJS({
          'ns2:abstract-common-list': {
            totalItems: totalItems,
            pageNum: searchDescriptor.searchQuery.p,
            pageSize: searchDescriptor.searchQuery.size
          }
        });
      }
    }
  }

  var searchState = _immutable2.default.fromJS({
    result: result,
    descriptor: action.meta,
    isPending: true
  });

  return updatedState.set('mostRecentKey', key).setIn(['byKey', key], searchState);
};

var handleSearchFulfilled = function handleSearchFulfilled(state, action) {
  var key = searchKey(action.meta);

  if (state.hasIn(['byKey', key])) {
    return state.setIn(['byKey', key], state.getIn(['byKey', key]).set('isPending', false).set('result', _immutable2.default.fromJS(action.payload.data)).delete('error'));
  }

  return state;
};

var handleSearchRejected = function handleSearchRejected(state, action) {
  var key = searchKey(action.meta);

  if (state.hasIn(['byKey', key])) {
    return state.setIn(['byKey', key], state.getIn(['byKey', key]).set('isPending', false).set('error', _immutable2.default.fromJS(action.payload)).delete('result'));
  }

  return state;
};

exports.default = function () {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _immutable2.default.fromJS({ mostRecentKey: null, byKey: {} });
  var action = arguments[1];

  switch (action.type) {
    case _search.SEARCH_STARTED:
      return handleSearchStarted(state, action);
    case _search.SEARCH_FULFILLED:
      return handleSearchFulfilled(state, action);
    case _search.SEARCH_REJECTED:
      return handleSearchRejected(state, action);
    default:
      return state;
  }
};

function isPending(state, searchDescriptor) {
  return state.getIn(['byKey', searchKey(searchDescriptor), 'isPending']);
}

function getResult(state, searchDescriptor) {
  return state.getIn(['byKey', searchKey(searchDescriptor), 'result']);
}

function getError(state, searchDescriptor) {
  return state.getIn(['byKey', searchKey(searchDescriptor), 'error']);
}